// Customparam
// Minimal machine that chooses the number of parameters at runtime according to its name
// for example rename the DLL to customparam_12.dll to have 12 dynamic parameters.

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "MachineInterface.h" // Modified for custom number of parameters
#include <vector>

#define MAX_TRACKS				8
#define NUMBER_OF_NORMAL_GLOBAL_PARAMS 1

// non-dynamic global parameters:
//									type        name         description     min  max     no     flags
CMachineParameter const paraG1 = { pt_word, "Global par. 1", "global param 1", 0, 65534, 65535, MPF_STATE, 0 };

// track parameters:
CMachineParameter const paraT1 = { pt_word, "Track par. 1", "track param 1", 0, 65534, 65535, MPF_STATE, 0 };

// this will be sent to Buzz as an array since vectors are binary-compatible:
static std::vector<CMachineParameter*> pParameters;

// these things will need to be stored on the heap:
static std::vector<CMachineParameter> parameters;

// number of dynamic params, will be set on load:
int nDynamic;

#pragma pack(1)

class gvals
{
public:
	word g1;
	byte gDynamic[4096]; // this could be unlimited and dynamic but im too lazy atm (alignment?)
};

class tvals
{
public:
	word t1;
};

#pragma pack()

CMachineInfo MacInfo = 
{
	MT_GENERATOR,				// type
	MI_VERSION,
	MIF_NO_OUTPUT,				// flags
	1,							// min tracks
	MAX_TRACKS,					// max tracks
	0,							// numGlobalParameters (will be set at runtime)
	1,							// numTrackParameters
	NULL,						// parameter definitions (will be set at runtime)
	0, 
	NULL,
	"customparam",
	"customparam",				// short name
	"pac", 						// author
	NULL
};

// stuff for getting the current DLL name:
EXTERN_C IMAGE_DOS_HEADER __ImageBase;

static void initParams()
{
	// get DLL name
	CHAR DllPath[MAX_PATH];
	CHAR DllName[MAX_PATH];
	GetModuleFileName((HINSTANCE)&__ImageBase, DllPath, _countof(DllPath));
	_splitpath(DllPath, NULL, NULL, DllName, NULL);

	// determine number of dynamic parameters from name
	sscanf(DllName, "customparam_%d", &nDynamic);

	parameters.push_back(paraG1); // non-dynamic param

	// dynamic params:
	for (int i=0; i<nDynamic; i++)
	{
		// allocate parameter names and decriptions on the heap
		std::vector<char> *name = new std::vector<char>(255, '\0');
		std::vector<char> *desc = new std::vector<char>(255, '\0');
		sprintf(&(*name)[0], "dynamic  %d", i); // todo safety
		sprintf(&(*desc)[0], "global dyn %d (1=on, 0=off)", i); // stay safe kids lol
		CMachineParameter mp = { 
			pt_switch,			// type
			&(*name)[0],		// name
			&(*desc)[0],		// description
			-1, 				// MinValue	
			-1,			  		// MaxValue
			SWITCH_NO,    		// NoValue
			MPF_STATE,			// Flags
			0
		};
		parameters.push_back(mp);
	}

	parameters.push_back(paraT1); // track param

	// buzz wants an array of pointers
	for (int i=0; i<parameters.size(); i++)
		pParameters.push_back(&(parameters[i]));
	
	MacInfo.numGlobalParameters = 1 + nDynamic;
	MacInfo.Parameters = &pParameters[0];
}


class mi : public CMachineInterface
{
public:
	mi();
	virtual ~mi();

	virtual void Init(CMachineDataInput * const pi);
	virtual void Tick();

	virtual bool Work(float *psamples, int numsamples, int const mode);

public:
	tvals tval[MAX_TRACKS];
	gvals gval;
	CMachine *pMac;
};

DLL_EXPORTS

mi::mi()
{
	GlobalVals = &gval;
	TrackVals  = tval;
	AttrVals   = NULL;
	pMac = 0;
}

mi::~mi()
{
}


void mi::Init(CMachineDataInput * const pi)
{
	pMac = pCB->GetThisMachine();
}

void mi::Tick()
{
	// let's check if everything works!
	char debugString[1024];
	if (gval.g1 != paraG1.NoValue) {
		sprintf(debugString, "G1 changed: %d", (int)gval.g1);
		pCB->WriteLine(debugString);
	}
	for (int i=0; i<nDynamic; i++)
	{
		if (gval.gDynamic[i] != parameters[i+NUMBER_OF_NORMAL_GLOBAL_PARAMS].NoValue) {
			sprintf(debugString, "GDynamic_%d changed: %d", i, (int)gval.gDynamic[i]);
			pCB->WriteLine(debugString);
		}
	}
}


bool mi::Work(float *psamples, int numsamples, int const mode) 
{ 
	return false; 
}

BOOL WINAPI DllMain (HANDLE hModule, DWORD fwdreason, LPVOID lpReserved) {
	switch (fwdreason) {
		case DLL_PROCESS_ATTACH:
			initParams();
			break;
		case DLL_THREAD_ATTACH: break;
		case DLL_THREAD_DETACH: break;
		case DLL_PROCESS_DETACH: break;
	}
	return TRUE;
}
